/**
 * @file
 * Constructs all the video players.
 */

((Drupal, Video) => {
  class DrupalVideo extends Video {
    /**
     * Constructs a Video player.
     *
     * @param {HTMLElement|string} $el
     *   Given selector or HTMLElement.
     * @param {Array} [options]
     *   Different options.
     */
    constructor($el, options) {
      super($el, {
        ...{
          plugins: {
            ad: {
              $handler: '.m-video__ad',
              activeClass: 'm-video__ad--is-active',
            },
            fullscreen: {
              $handler: '.m-video__fullscreen',
              fullscreenClass: 'm-video--is-fullscreen',
              activeClass: 'm-video__fullscreen--is-fullscreen',
            },
            mute: {
              $handler: '.m-video__mute',
              muteClass: 'm-video__mute--is-muted',
            },
            loading: {
              $el: '.m-video__throbber',
              showClass: 'm-video__throbber--is-shown',
            },
            play: {
              $handler: '.m-video__play',
              showClass: 'm-video__play--is-shown',
            },
            playpause: {
              $handler: '.m-video__playpause',
              playClass: 'm-video__playpause--is-playing',
            },
            progress: {
              $el: '.m-video__progress',
              $bar: '.m-video__progress-bar',
              $loaded: '.m-video__progress-loaded',
            },
            caption: {
              $handlers: '.m-video__caption',
              activeClass: 'm-video__caption--is-active',
            },
            time: {
              $currentTime: '.m-video__current-time',
              $duration: '.m-video__duration',
            },
            volume: {
              $el: '.m-video__volume',
              $bar: '.m-video__volume-bar',
            },
          },
        },
        ...options,
      });

      // Handle play.
      this.on(this.$el, 'play.video', () => {
        const $playpause = this.plugins.playpause.$handler;

        this.$el.classList.add('m-video--is-playing');
        $playpause.setAttribute('aria-label', $playpause.dataset.pauseLabel);
      });

      // Handle pause.
      this.on(this.$el, 'pause.video', () => {
        const $playpause = this.plugins.playpause.$handler;

        this.$el.classList.remove('m-video--is-playing');
        $playpause.setAttribute('aria-label', $playpause.dataset.playLabel);
      });

      // Handle volume change.
      this.on(this.$el, 'volumechange.video', (e) => {
        const $mute = this.plugins.mute.$handler;

        $mute.setAttribute(
          'aria-label',
          $mute.dataset[`mute${e.detail.muted ? 'Off' : 'On'}Label`],
        );
      });

      // Handle caption.
      this.on(this.$el, 'caption.video', (e) => {
        e.detail.$handlers.forEach(($handler) => {
          $handler.setAttribute(
            'aria-label',
            $handler.dataset[
              `caption${$handler.track.mode === 'showing' ? 'Off' : 'On'}Label`
            ],
          );
        });
      });

      // Handle audio description.
      this.on(this.$el, 'ad.video', (e) => {
        const $ad = this.plugins.ad.$handler;

        $ad.setAttribute(
          'aria-label',
          $ad.dataset[`ad${e.detail.state === 'on' ? 'Off' : 'On'}Label`],
        );
      });

      // Handle fullscreen.
      this.on(this.$el, 'fullscreen.video', (e) => {
        const $fullscreen = this.plugins.fullscreen.$handler;

        $fullscreen.setAttribute(
          'aria-label',
          $fullscreen.dataset[
            `fullscreen${e.detail.fullscreen ? 'Exit' : 'Enter'}Label`
          ],
        );
      });
    }
  }

  Drupal.Video = DrupalVideo;

  /**
   * Attaches the video behavior.
   *
   * @type {Drupal~behavior}
   *
   * @prop {Drupal~behaviorAttach} attach
   *   Attaches the behavior.
   */
  Drupal.behaviors.video = {
    attach(context) {
      const $videos = once('video', '.js-video', context.parentNode || context);

      $videos.forEach(($video) => {
        // eslint-disable-next-line no-unused-vars
        const video = new Drupal.Video($video);
      });
    },
  };
})(Drupal, Video, once);
